@@ -1,6 +1,6 @@
 package swat;
 
-our $VERSION = '0.1.38';
+our $VERSION = '0.1.39';
 
 use base 'Exporter'; 
 
@@ -20,7 +20,7 @@ use Test::More;
 our $HTTP_RESPONSE;
 our ($project);
 our ($curl_cmd, $content_file);
-our ($url, $path, $http_meth); 
+our ($url, $path, $route_dir, $http_meth); 
 our ($debug, $ignore_http_err, $try_num, $debug_bytes);
 our ($is_swat_package);
 $| = 1;
@@ -885,7 +885,7 @@ Route based settings are applied if exist.
 
 =item *
 
-And finally environment settings aer applied if exist.
+And finally environment settings are applied if exist.
 
 =back
 
@@ -918,16 +918,20 @@ Bash Hooks
 
 =head2 Perl hooks
 
-Perl hooks are just perl code files `required` in the beginning of next swat test. There are 2 types of perl hooks:
+Perl hooks are files with perl code `required` I<in the beginning/end of a swat test>. There are four types of perl hooks:
 
 =over 
 
 =item *
 
-B<project based hook>
+B<project based perl startup hook>
 
-File located at C<$project_root_directory/hook.pm>. Project based hooks are applied for every route in project and
-could be used for I<project initialization>. For example one could define generators here:
+File located at C<$project_root_directory/hook.pm>. 
+
+Project based startup hooks are `required` I<in the begining> of a swat test and applied for every route in project 
+and thus could be used for I<project initialization> procedures. 
+
+For example one could define common generators here:
 
     # place this in hook.pm file:
     sub list1 { | %w{ foo bar baz } | }
@@ -940,13 +944,26 @@ could be used for I<project initialization>. For example one could define genera
 
 =item *
 
-B<route based hooks>
+B<project based perl cleanup hook>
+
+File located at C<$project_root_directory/cleanup.pm>. 
+
+This hooks is similar to startup hook but `required` I<in the end> of a swat test.
+
+=item *
+
+B<route based perl startup hooks>
+
+Files located at C<$route_directory/hook.pm>. 
+
+Routes based startup hooks are applied for every route in project and thus could be used for I<route initialization> procedures.
+
+For example one could define route specific generators here:
 
-Files located at C<$project_root_directory/$route_directory/hook.pm>. Routes based hook are route specific hooks and
-could be used for I<route initialization>. For example one could define route specific generators here:
 
     # place this in hook.pm file:
-    # notices that we could tell GET from POST http methods here:
+    # notices that we could tell GET from POST http methods here
+    # using predefined $method variable
 
     sub list1 { 
 
@@ -967,30 +984,56 @@ could be used for I<route initialization>. For example one could define route sp
     generator:  list() 
 
 
+=item *
+
+B<route based perl cleanup hooks>
+
+Files located at C<$route_directory/cleanup.pm>.
+
+This hooks is similar to route based startup hooks but `required` I<in the end> of a swat test.
 
 =back
 
 =head2 Bash hooks
 
-Similar to perl hooks bash hooks are just a bash files `sourced` I<before> execution of next swat test. 
+Similar to perl hooks bash hooks are just a bash files `sourced` I<before compilation> of a swat test. 
 
-There are 2 types of bash hooks:
+There are 4 types of bash hooks:
 
 =over 
 
 =item *
 
-B<project based hook>
+B<project based bash hook>
 
-File located at C<$project_root_directory/hook.bash>. Project based hooks are applied for every route in project and
-could be used for I<project initialization>.
+File located at C<$project_root_directory/hook.bash>. 
+
+Project based bash hooks are applied for every route in project and could be used for I<project initialization> procedures.
 
 =item *
 
-B<route based hooks>
+B<route based bash hooks>
 
-Files located at C<$project_root_directory/$route_directory/hook.bash>. Routes based hook are route specific hooks and
-could be used for I<route initialization>.
+Files located at C<$project_root_directory/$route_directory/hook.bash>. 
+
+Routes based bash hooks are route specific hooks and could be used for I<route initialization> procedures.
+
+=item *
+
+B<global startup bash hook>
+
+File located at C<$project_root_directory/startup.bash>. 
+
+Startup hook is executed before swat tests gets compiled, at the very begining, at could be used for I<global initialization> procedures.
+
+
+=item *
+
+B<global cleanup bash hook>
+
+File located at C<$project_root_directory/cleanup.bash>. 
+
+Cleanup hook is executed I<after swat tests are executed>, at the very end, and could be used for I<global cleanup> procedures.
 
 =back
 
@@ -998,7 +1041,9 @@ It is important to note that bash hooks are executed I<after swat settings merge
 about swat settings.
 
 
-Useful environment vairables one may rely upon when writting bash hooks:
+=head2 Predifined variables 
+
+List of variables one may rely upon when writting perl/bash hooks:
 
 =over 
 
@@ -1014,9 +1059,38 @@ B<curl_params>
 
 B<http_meth> - C<GET|POST>
 
+=item *
+
+B<route_dir>
+
+=item *
+
+B<project>
+
 =back
 
 
+=head1 Swat Compile and Runtime 
+
+ - Execute *global startup bash hook*
+ - Start of swat compilation phase
+ - For every route gets compiled:
+    -- Merge swat settings
+     -- Set predifined variables
+     -- Execute *project based bash hook*
+     -- Execute *route based bash hook*
+     -- Compile route test
+ - The end of swat compilation phase
+ - Start of swat executation phase. 
+ - For every route test gets executed:
+     -- Execute *project based perl startup hook*
+     -- Execute *route based perl startup hook*
+     -- Execute route test
+     -- Execute *route based perl cleanup hook*
+     -- Execute *project based perl cleanup hook*
+ - The end of swat compilation phase
+ - Execute *global cleanup bash hook*
+
 =head1 TAP
 
 Swat produces output in L<TAP|https://testanything.org/> format , that means you may use your favorite tap parsers to bring result to
